@@ -1,27 +1,33 @@
+require 5.002;	# 5.002beta2 or later
+
 {
 package DBI;
 
-require 5.001;	# 5.001m or later
+$VERSION = '0.67';
 
-$VERSION = substr(q$Revision: 1.47 $, 10);
+my $Revision = substr(q$Revision: 1.51 $, 10);
 
-# $Id: DBI.pm,v 1.47 1995/10/24 02:20:41 timbo Exp $
+# $Id: DBI.pm,v 1.51 1996/02/15 22:43:55 timbo Exp $
 #
 # Copyright (c) 1995, Tim Bunce
 #
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
 
-require DynaLoader;
-@ISA = qw(DynaLoader);
+use Carp;
+use DynaLoader ();
+use Exporter ();
+
+@ISA = qw(Exporter DynaLoader);
+
+# Make some utility functions available if asked for
+@EXPORT_OK = qw(neat neat_list dump_results);
+@EXPORT    = qw();	# Export _nothing_ by default
 
 use strict;
-use Carp;
-
-sub import { }	# workaround Perl5.001i AutoLoader problems
 
 $DBI::dbi_debug = $ENV{PERL_DBI_DEBUG} || 0;
-warn "IN DBI.pm\n" if $DBI::dbi_debug;
+carp "Loaded DBI.pm" if $DBI::dbi_debug;
 
 bootstrap DBI;
 
@@ -33,6 +39,7 @@ DBI->_debug_dispatch($DBI::dbi_debug) if $DBI::dbi_debug;
 # Setup special DBI dynamic variables. See DBI::var::FETCH for details.
 # These are dynamically associated with the last handle used.
 tie $DBI::err,    'DBI::var', '*err';    # special case: referenced via IHA list
+tie $DBI::state,  'DBI::var', '"state';  # special case: referenced via IHA list
 tie $DBI::lasth,  'DBI::var', '!lasth';  # special case: return boolean
 tie $DBI::errstr, 'DBI::var', '&errstr'; # call &errstr in last used pkg
 tie $DBI::rows,   'DBI::var', '&rows';   # call &rows   in last used pkg
@@ -43,20 +50,24 @@ sub DBI::var::STORE{ Carp::carp "Can't modify \$DBI::${$_[0]} special variable"
 # --- Dynamically create the DBI Standard Interface
 
 my $std = undef;
+my $keeperr = { O=>0x04 };
 
 my @TieHash_IF = (	# Generic Tied Hash Interface
-	'STORE'   => $std, 'FETCH'   => $std,
-	'FIRSTKEY'=> $std, 'NEXTKEY' => $std,
-	'EXISTS'  => $std, 'CLEAR'   => $std,
-	'DESTROY' => $std,
+	'STORE'   => $std,
+	'FETCH'   => $std,
+	'FIRSTKEY'=> $keeperr,
+	'NEXTKEY' => $keeperr,
+	'EXISTS'  => $keeperr,
+	'CLEAR'   => $keeperr,
+	'DESTROY' => $keeperr,
 );
 my @Common_IF = (	# Interface functions common to all DBI classes
-	func    =>	{ O => 0x02	},
-	event   =>	{ U =>[2,3,'$message, $retvalue'] },
-	debug   =>	{ U =>[1,2,'[$debug_level]'] },
-	private_data =>	{ U =>[1,1] },
-	errstr  =>	$std,
-	rows    =>	$std,
+	func    =>	{					O=>0x02	},
+	event   =>	{ U =>[2,3,'$message, $retvalue'],	O=>0x04 },
+	debug   =>	{ U =>[1,2,'[$debug_level]'],		O=>0x04 },
+	private_data =>	{ U =>[1,1],				O=>0x04 },
+	errstr  =>	$keeperr,
+	rows    =>	$keeperr,
 );
 
 my %DBI_IF = (	# Define the DBI Interface:
@@ -64,6 +75,7 @@ my %DBI_IF = (	# Define the DBI Interface:
     dr => {		# Database Driver Interface
 	'connect'  =>	{ U =>[1,5,'[$db [,$user [,$passwd [,\%attr]]]]'] },
 	disconnect_all=>{ U =>[1,1] },
+	data_sources => { U =>[1,1] },
 	@Common_IF,
 	@TieHash_IF,
     },
@@ -73,9 +85,10 @@ my %DBI_IF = (	# Define the DBI Interface:
 	'do'       =>	{ U =>[2,3,'$statement [, \%attribs]'] },
 	prepare    =>	{ U =>[2,3,'$statement [, \%attribs]'] },
 	handler    =>	{ U =>[2,2,'\&handler'] },
-	errstate   =>	{ U =>[1,1] },
-	errmsg     =>	{ U =>[1,1] },
+	errstate   =>	{ U =>[1,1], O=>0x04 },
+	errmsg     =>	{ U =>[1,1], O=>0x04 },
 	disconnect =>	{ U =>[1,1] },
+	tables     =>	{ U =>[1,1] },
 	@Common_IF,
 	@TieHash_IF,
     },
@@ -87,6 +100,8 @@ my %DBI_IF = (	# Define the DBI Interface:
 	fetch      =>	undef, # no checks, no args, max speed
 	fetchrow   =>	undef, # no checks, no args, max speed
 	readblob   =>	{ U =>[4,5,'$field, $offset, $len [, \\$buf [, $bufoffset]]'] },
+	blob_read  =>	{ U =>[4,5,'$field, $offset, $len [, \\$buf [, $bufoffset]]'] },
+	blob_copy_to_file => { U =>[3,3,'$field, $filename_or_handleref'] },
 	finish     => 	{ U =>[1,1] },
 	@Common_IF,
 	@TieHash_IF,
@@ -133,7 +148,7 @@ sub connect {
     my $drh = $DBI::installed_drh{$driver};
     unless (defined $drh){
 	$drh = DBI->install_driver($driver, $attr)
-		or confess "install_driver($driver) failed";
+		or confess "DBI->install_driver($driver) failed";
     }
     warn "DBI->connect using $driver driver $drh\n" if $DBI::dbi_debug;
 
@@ -226,11 +241,28 @@ sub available_drivers {
 }
 
 
-sub default_handler {	# will migrate to xs later
-    my($h, $type, @args) = @_;
-    Carp::carp "$type EVENT: $h->DBI::default_handler:\n\t@args\n";
-#   DBI::set_err($h, $args[0]||1);	# HACK, NOW CLASHES WITH Oracle/Oraperl
-    undef;
+sub neat_list {
+    my($listref, $maxlen, $sep) = @_;
+    $maxlen = 0 unless defined $maxlen;
+    $sep = ", " unless defined $sep;
+    join(", ", map { neat($_,$maxlen) } @$listref);
+}
+
+
+sub dump_results {
+    my($sth, $maxlen, $sep) = @_;
+    $maxlen ||= 35;
+    $sep    ||= "\n";
+    my $rows = 0;
+    my $ref;
+#$sth->debug(2);
+    while($ref = $sth->fetch) {
+	print $sep if $rows++;
+	print neat_list($ref,$maxlen);
+    }
+#$sth->debug(0);
+    print "\n$rows rows".($DBI::err ? " ($DBI::err: $DBI::errstr)" : "")."\n";
+    $rows;
 }
 
 
@@ -291,11 +323,14 @@ sub _new_handle {
 
 sub _new_drh {	# called by DBD::<drivername>::driver()
     my($class, $initial_attr, $imp_data) = @_;
-    my($h_err_store, $h_errstr_store) = (0, '');
+    # Provide default storage for State,Err and Errstr.
+    # State must be undef to get automatic faking in DBI::var::FETCH
+    my($h_state_store, $h_err_store, $h_errstr_store) = (undef, 0, '');
     my $attr = {
 	'ImplementorClass' => $class,
 	# these attributes get copied down to child handles
 	'Handlers'	=> [],
+	'State'		=> \$h_state_store,  # Holder for DBI::state
 	'Err'		=> \$h_err_store,    # Holder for DBI::err
 	'Errstr'	=> \$h_errstr_store, # Holder for DBI::errstr
 	'Debug' 	=> 0,
@@ -357,7 +392,7 @@ sub _new_sth {	# called by DBD::<drivername>::db::prepare()
 		'Name'    => 'Switch',
 		'Version' => $DBI::VERSION,
 		# the Attribution is defined as a sub as an example
-		'Attribution' => sub { "DBI Switch by Tim Bunce" },
+		'Attribution' => sub { "DBI-$DBI::VERSION Switch by Tim Bunce" },
 	    }, \$err);
 	Carp::confess("DBD::Switch init failed!") unless ($drh && $inner);
 	$DBD::Switch::dr::drh;
@@ -459,6 +494,30 @@ sub _new_sth {	# called by DBD::<drivername>::db::prepare()
     sub rows	{ -1 }
     # bind_param => not implemented - fail
 
+    sub readblob {		# grandfather in for a version or two
+	my $count if 0;		# trick to get static variable
+	Carp::carp("Warning: readblob method renamed to blob_read()")
+	    unless $count++;
+	shift->blob_read(@_);
+    }
+    sub blob_copy_to_file {	# returns length or undef on error
+	my($self, $field, $filename_or_handleref, $blocksize) = @_;
+	my $fh = $filename_or_handleref;
+	my($len, $buf) = (0, "");
+	$blocksize ||= 512;	# not too ambitious
+	local(*FH);
+	unless(ref $fh) {
+	    open(FH, ">$fh") || return undef;
+	    $fh = \*FH;
+	}
+	while(defined($self->blob_read($field, $len, $blocksize, \$buf))) {
+	    print $fh $buf;
+	    $len += length $buf;
+	}
+	close(FH);
+	$len;
+    }
+
     # Drivers are required to implement *::st::DESTROY to encourage tidy-up
     sub DESTROY  { Carp::confess "Driver has not implemented DESTROY for @_" }
 }
@@ -540,15 +599,14 @@ $rc  = $dbh->disconnect;			undef or 1
 $rc  = $dbh->commit;				undef or 1
 $rc  = $dbh->rollback;				undef or 1
 $rc  = $dbh->do($statement [, \%attr]);
-$sth = $dbh->prepare($statement [, \%attr]);
 
-$rv  = $dbh->handler($handler_function);
+$sth = $dbh->prepare($statement [, \%attr]);
+$sth = $dbh->tables();
+
 $rv  = $dbh->errstate;
 @ary = $dbh->errmsg;
 
 $sql = $dbh->quote($str);
-$udt = $dbh->ndt2udt($ndt [, $local]);
-$ndt = $dbh->udt2ndt($udt [, $local]);
 
 $dbh->{Type}       "db"
 $dbh->{Name}       (name of database the handle is connected to)
@@ -580,3 +638,9 @@ $sth->{NUM_OF_FIELDS}  ($)
 $sth->{NUM_OF_PARAMS}  ($)
 
 ---------------------------------------------------------------
+
+WWW links
+
+http://www-ccs.cs.umass.edu/db.html
+http://www.odmg.org/odmg93/updates_dbarry.htm
+http://www.jcc.com/sql_stnd.html
